% File name: mainbisection_2p_for_simulation.m
%   Ver. 0.1 (26-Jul-2011)
%%%%%%%%%%
% Before you run this program, you need to make a directory whose
% name is "Results_dir".
%%%%%%%%%%
% This program is for Monte-Carlo simulation in the parameter 
% estimation of the three-parameter lognormal distribution.
% The algorithm proposed in the paper [1] is used for the estimation. 
% In order to simulate data, the same function as that in the paper [2]
% is used. If you replace the function "setdata" in the program with 
% your function, you can get your simulation results.
%
% [References]
% [1] Y. Komori and H. Hirose (2004), Easy estimation by a new
%     parameterization for the three-parameter lognormal distribution,
%     Journal of Statistical Computation and Simulation, 74 (1), 63-74.
% [2] J. F. Vera and J. A. Diaz-Garcia (2008), A global simulated
%     annealing heuristic for the three-parameter lognormal maximum
%     likelihood estimation, Computational Statistical and Data
%     Analysis, 52, 5055-5065.
%%%%%%%%%%
% Check for a directory to put output files
dname='Results_dir';
if 7~=exist('Results_dir','dir')
    error(strcat('Make the directory "',dname,'"'));
end
% Initialization for a data set
lambda=2; % 0.25, 0.5, 0.75, 1, 1.25, 1.5, 1.75 or 2.
initstate=0; % from 0 to 2^32-1
ndata=15; % 10, 15.
% See p. 64 and p. 71 in [1]
al=-1/lambda; 
lnGam=-log(lambda);
randn('state',initstate);
%%%
eps0=0.05;
eps1=1e-14;
eps2=1e-2;
dLam=0.05;
i_lam_max=1000;
%%%
num_batch=16; % 16
num_sim=1000; % 1000
%%%
record_plus=zeros(i_lam_max,3);
record_minus=record_plus;
record_all=zeros(i_lam_max*2,3);
max_triplet=zeros(10,3);
%%%
for batch_i=1:num_batch
    %
    positive_cnt=0;
    sim_exist_cnt=0;
    for sim_i=1:num_sim
        % Data set part --- begin ---
        cur_state=randn('state');
        Z=randn(ndata,1);
        tmp=lnGam+lambda*Z;
        datax=exp(tmp)+al;
        % Data set part --- end ---
        %
        datax=sort(datax,'descend');
        xmax=datax(1);
        xmin=datax(length(datax));
        xmean=mean(datax);
        %
        tau_ast=0;
        for ii=1:ndata-1
            tmp=datax(ii+1:ndata)-datax(ii);
            tau_ast=tau_ast+sum(tmp.*tmp);
        end
        tau_ast=sqrt(tau_ast)/ndata;
        %
        break_flag_plus=0;
        % Search in the region lambda > 0
        tau_tmp=tau_ast;
        for i_lam=1:i_lam_max
            lam_tmp=eps0+dLam*(i_lam-1);
            tau_min=-lam_tmp*xmin;
            tmp=exp(-lam_tmp*lam_tmp);
            tau_max=tau_min*(1-xmean/xmin*tmp)/(1-tmp);
            if (tau_min<tau_tmp) && (tau_tmp<tau_max)
                [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
                if 0>errflag
                    return;
                elseif 1==errflag
                    break_flag_plus=1;
                    break;
                end
                if 0<val
                    tau_min=tau_tmp;
                else
                    tau_max=tau_tmp;
                end
            end
            %
            while (eps1 < (tau_max-tau_min)/abs(tau_max))
                [val,errflag]=dF_dTau(datax, lam_tmp, (tau_min+tau_max)/2);
                if 0>errflag
                    return;
                elseif 1==errflag
                    break_flag_plus=1;
                    break;
                end
                if 0<val
                    tau_min=(tau_min+tau_max)/2;
                else
                    tau_max=(tau_min+tau_max)/2;
                end
            end
            %   
            tau_tmp=tau_max;
            [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
            if 0>errflag
                return;
            elseif 1==errflag
                break_flag_plus=1;
                break;
            end
            if eps2>abs(val)
                record_plus(i_lam,:)=[lam_tmp tau_tmp funcF(datax,lam_tmp,tau_tmp)];
            else
                break_flag_plus=2;
                break;
            end
        end
        if i_lam_max==i_lam
            break_flag_plus=3;
        end
        i_lam_max_plus1=i_lam-1;
        %
        break_flag_minus=0;
        % Search in the region lambda < 0
        tau_tmp=tau_ast;
        for i_lam=1:i_lam_max
            lam_tmp=-eps0-dLam*(i_lam-1);
            tau_min=-lam_tmp*xmax;
            tmp=exp(-lam_tmp*lam_tmp);
            tau_max=tau_min*(1-xmean/xmax*tmp)/(1-tmp);
            if (tau_min<tau_tmp) && (tau_tmp<tau_max)
                [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
                if 0>errflag
                return;
                elseif 1==errflag
                    break_flag_plus=1;
                    break;
                end
                if 0<val
                    tau_min=tau_tmp;
                else
                    tau_max=tau_tmp;
                end
            end
            %
            while (eps1 < (tau_max-tau_min)/abs(tau_max))
                [val,errflag]=dF_dTau(datax, lam_tmp, (tau_min+tau_max)/2);
                if 0>errflag
                    return;
                elseif 1==errflag
                    break_flag_plus=1;
                    break;
                end
                if 0<val
                    tau_min=(tau_min+tau_max)/2;
                else
                    tau_max=(tau_min+tau_max)/2;
                end
            end
            %
            tau_tmp=tau_max;
            [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
            if 0>errflag
                return;
            elseif 1==errflag
                break_flag_plus=1;
                break;
            end
            if eps2>abs(val)
                record_minus(i_lam,:)=[lam_tmp tau_tmp funcF(datax,lam_tmp,tau_tmp)];
            else
                break_flag_minus=2;
                break;
            end
        end
        if i_lam_max==i_lam
            break_flag_minus=3;
        end
        i_lam_max_minus1=i_lam-1;
        %
        % Summarizing the record
        for ii=1:i_lam_max_minus1
            record_all(ii,:)=record_minus(i_lam_max_minus1+1-ii,:);
        end
        for ii=1:i_lam_max_plus1
            record_all(i_lam_max_minus1+ii,:)=record_plus(ii,:);
        end
        i_lam_max_all=i_lam_max_minus1+i_lam_max_plus1;
        %
        % Search for the local maximum
        i_max_triplet=0;
        up_down_flag=0;
        f_tmp=record_all(1,3);
        for ii=2:i_lam_max_all
            if f_tmp>record_all(ii,3)
                if 1==up_down_flag
                    i_max_triplet=i_max_triplet+1;
                    max_triplet(i_max_triplet,:)=[record_all(ii-1,1:2) f_tmp];
                    up_down_flag=-1;
                end
                up_down_flag=-1;
            else
                up_down_flag=1;
            end
            f_tmp=record_all(ii,3);
        end
        %
        mle_triplet=max_triplet(1,:);
        for ii=2:i_max_triplet
            if mle_triplet(3)<max_triplet(ii,3)
                mle_triple=max_triplet(ii,:);
            end
        end
        %
        if 0~=i_max_triplet
            sim_exist_cnt=sim_exist_cnt+1;
            if 0<mle_triplet(1)
                positive_cnt=positive_cnt+1;
            end
        else
            fstate=cur_state;
        end
        %
    end
    %
    fname=strcat(dname,'\res',num2str(batch_i),'_L',num2str(lambda),'_n',num2str(ndata),'.txt');
    fid=fopen(fname,'w');
    if 0~=sim_exist_cnt
        fprintf(fid, '(lambda, ndata, nsim, ex_rate, p_rate) = (%f, %d, %d, %f, %f)\n',lambda, ndata, num_sim, sim_exist_cnt/num_sim, positive_cnt/sim_exist_cnt);
    else
        fprintf(fid, '(lambda, ndata, nsim, ex_rate) = (%f, %d, %d, %f)\n',lambda, ndata, num_sim, sim_exist_cnt/num_sim);
    end
    fclose(fid);
end