% Filename: mainbisection_2p_for_single_data_set.m
%   Ver. 0 (9-Dec-2010)
%%%
% This program is used for a parameter estimation of the three-parameter
% lognormal distribution. When you replace the data set part with 
% another one to give your data, you will obtain a parameter estimate 
% for the data if a local maximum exists. If it does not exist, this 
% program will tell you that.
%%%
% Initialization for a data set
lambda=2;
initstate=0; % from 0 to 2^32-1 or fstate, which is set in main...simulation.m
ndata=20;
%%%
eps0=0.01;
eps1=1e-14;
eps2=1e-2;
dLam=0.01;
i_lam_max=1000;
%%%
record_plus=zeros(i_lam_max,3);
record_minus=record_plus;
record_all=zeros(i_lam_max*2,3);
max_triplet=zeros(10,3);
%%%
% Data set part --- begin ---
% See p. 64 and p. 71 in [1]
al=-1/lambda; 
lnGam=-log(lambda);
randn('state',initstate);
Z=randn(ndata,1);
tmp=lnGam+lambda*Z;
datax=exp(tmp)+al;
% Data set part --- end ---
%
datax=sort(datax,'descend');
xmax=datax(1);
xmin=datax(length(datax));
xmean=mean(datax);
tau_ast=0;
for ii=1:ndata-1
    tmp=datax(ii+1:ndata)-datax(ii);
    tau_ast=tau_ast+sum(tmp.*tmp);
end
tau_ast=sqrt(tau_ast)/ndata;
%
break_flag_plus=0;
% Search in the region lambda > 0
tau_tmp=tau_ast;
for i_lam=1:i_lam_max
    lam_tmp=eps0+dLam*(i_lam-1);
    tau_min=-lam_tmp*xmin;
    tmp=exp(-lam_tmp*lam_tmp);
    tau_max=tau_min*(1-xmean/xmin*tmp)/(1-tmp);
    if (tau_min<tau_tmp) && (tau_tmp<tau_max)
        [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
        if 0>errflag
            return;
        elseif 1==errflag
            break_flag_plus=1;
            break;
        end
        if 0<val
            tau_min=tau_tmp;
        else
            tau_max=tau_tmp;
        end
    end
    
    while (eps1 < (tau_max-tau_min)/abs(tau_max))
        [val,errflag]=dF_dTau(datax, lam_tmp, (tau_min+tau_max)/2);
        if 0>errflag
            return;
        elseif 1==errflag
            break_flag_plus=1;
            break;
        end
        if 0<val
            tau_min=(tau_min+tau_max)/2;
        else
            tau_max=(tau_min+tau_max)/2;
        end
    end
    
    tau_tmp=tau_max;
    [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
    if 0>errflag
        return;
    elseif 1==errflag
        break_flag_plus=1;
        break;
    end
    if eps2>abs(val)
        record_plus(i_lam,:)=[lam_tmp tau_tmp funcF(datax,lam_tmp,tau_tmp)];
    else
        break_flag_plus=2;
        break;
    end
end
if i_lam_max==i_lam
    break_flag_plus=3;
end
i_lam_max_plus1=i_lam-1;

break_flag_minus=0;
% Search in the region lambda < 0
tau_tmp=tau_ast;
for i_lam=1:i_lam_max
    lam_tmp=-eps0-dLam*(i_lam-1);
    tau_min=-lam_tmp*xmax;
    tmp=exp(-lam_tmp*lam_tmp);
    tau_max=tau_min*(1-xmean/xmax*tmp)/(1-tmp);
    if (tau_min<tau_tmp) && (tau_tmp<tau_max)
        [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
        if 0>errflag
            return;
        elseif 1==errflag
            break_flag_plus=1;
            break;
        end
        if 0<val
            tau_min=tau_tmp;
        else
            tau_max=tau_tmp;
        end
    end
    
    while (eps1 < (tau_max-tau_min)/abs(tau_max))
        [val,errflag]=dF_dTau(datax, lam_tmp, (tau_min+tau_max)/2);
        if 0>errflag
            return;
        elseif 1==errflag
            break_flag_plus=1;
            break;
        end
        if 0<val
            tau_min=(tau_min+tau_max)/2;
        else
            tau_max=(tau_min+tau_max)/2;
        end
    end
    
    tau_tmp=tau_max;
    [val,errflag]=dF_dTau(datax, lam_tmp, tau_tmp);
    if 0>errflag
        return;
    elseif 1==errflag
        break_flag_plus=1;
        break;
    end
    if eps2>abs(val)
        record_minus(i_lam,:)=[lam_tmp tau_tmp funcF(datax,lam_tmp,tau_tmp)];
    else
        break_flag_minus=2;
        break;
    end
end
if i_lam_max==i_lam
    break_flag_minus=3;
end
i_lam_max_minus1=i_lam-1;

% Output of the profile
fp=fopen('profile_for_single_data_by_MATLAB','wt');
fprintf(fp, '%% break_flag_minus, plus = %d, %d\n', break_flag_minus, break_flag_plus);
for ii=i_lam_max_minus1:-1:1
    fprintf(fp, '%f\t%f\t%f\n', record_minus(ii,:));
end
for ii=1:i_lam_max_plus1
    fprintf(fp, '%f\t%f\t%f\n', record_plus(ii,:));
end
fclose(fp);

% Output of the edge
output_Edge(i_lam_max, eps0, dLam, datax, xmean, xmin, xmax);

% Summarizing the record
for ii=1:i_lam_max_minus1
    record_all(ii,:)=record_minus(i_lam_max_minus1+1-ii,:);
end
for ii=1:i_lam_max_plus1
    record_all(i_lam_max_minus1+ii,:)=record_plus(ii,:);
end
i_lam_max_all=i_lam_max_minus1+i_lam_max_plus1;

% Output of the local maximum
i_max_triplet=0;
up_down_flag=0;
f_tmp=record_all(1,3);
for ii=2:i_lam_max_all
    if f_tmp>record_all(ii,3)
        if 1==up_down_flag
            i_max_triplet=i_max_triplet+1;
            max_triplet(i_max_triplet,:)=[record_all(ii-1,1:2) f_tmp];
            up_down_flag=-1;
        end
        up_down_flag=-1;
    else
        up_down_flag=1;
    end
    f_tmp=record_all(ii,3);
end

for ii=1:i_max_triplet
    fprintf(1, 'max_triplet %d: (%f,%f,%f)\n', ii, max_triplet(ii,:));
end

mle_triplet=max_triplet(1,:);
for ii=2:i_max_triplet
    if mle_triplet(3)<max_triplet(ii,3)
        mle_triple=max_triplet(ii,:);
    end
end

if 0==i_max_triplet
    fprintf(1,'No mle\n');
    return;
end

fprintf(1,'F=%f\n', funcF(datax, mle_triplet(1), mle_triplet(2)));
fprintf(1, 'lambda=%f  ', mle_triplet(1));
fprintf(1, 'tau=%f    ', mle_triplet(2));
tmp_s=sum(log(mle_triplet(1)*datax+mle_triplet(2)))/ndata;
fprintf(1, 's    =%f\n', tmp_s);
fprintf(1, '\t\t\t\t ');
fprintf(1, 'mu =%f    ', (exp(tmp_s)-mle_triplet(2))/mle_triplet(1));
fprintf(1, 'sigma=%f\n', exp(tmp_s));
fprintf(1, 'alpha=%f  ', (exp(tmp_s)-mle_triplet(2))/mle_triplet(1)-exp(tmp_s)/mle_triplet(1));
fprintf(1, 'beta =%f  ', mle_triplet(1));
fprintf(1, 'gamma=%f\n', exp(tmp_s)/mle_triplet(1));
