% Filename: output_Edge.m
%  This function gives data concerning dotted curves indicated
%  in a figure of [1].
%
% [References]
% [1] Y. Komori, Robust algorithm associated with a parameterization
%     for the three-parameter lognormal distribution, submitted
%     for possible publication. 
%%%
function output_Edge(i_lam_max, eps0, dLam, datax, xmean, xmin, xmax)
  %
  aprox_zero=1e-14;
  %
  edge_plus=zeros(i_lam_max,3);
  edge_minus=edge_plus;
  %
  break_flag_plus=0;
  for i_lam=1:i_lam_max
      lam_tmp=eps0+dLam*(i_lam-1);
      tmp=exp(-lam_tmp*lam_tmp);
      tmp=(1-xmean/xmin*tmp)/(1-tmp);
      if 1 == tmp
          break_flag_plus=1;
          break;
      end
      tau_tmp=-lam_tmp*xmin*tmp;
      tmp=lam_tmp*datax+tau_tmp;      
      if 1~=isempty(find(tmp<=aprox_zero))
          break_flag_plus=2;
          break;
      end
      edge_plus(i_lam,:)=[lam_tmp tau_tmp funcF(datax,lam_tmp,tau_tmp)];
  end
  i_lam_max_plus1_edge=i_lam-1;
  
  break_flag_minus=0;
  for i_lam=1:i_lam_max
      lam_tmp=-eps0-dLam*(i_lam-1);
      tmp=exp(-lam_tmp*lam_tmp);
      tmp=(1-xmean/xmax*tmp)/(1-tmp);
      if 1 == tmp
          break_flag_minus=1;
          break;
      end
      tau_tmp=-lam_tmp*xmax*tmp;
      tmp=lam_tmp*datax+tau_tmp;      
      if 1~=isempty(find(tmp<=aprox_zero))
          break_flag_minus=2;
          break;
      end
      edge_minus(i_lam,:)=[lam_tmp tau_tmp funcF(datax,lam_tmp,tau_tmp)];
  end
  i_lam_max_minus1_edge=i_lam-1;
  
  fp=fopen('edge_for_single_data_by_MATLAB','wt');
  fprintf(fp, '%% break_flag_minus, plus = %d, %d\n', break_flag_minus, break_flag_plus);
  for ii=i_lam_max_minus1_edge:-1:1
      fprintf(fp, '%f\t%f\t%f\n', edge_minus(ii,:));
  end
  for ii=1:i_lam_max_plus1_edge
      fprintf(fp, '%f\t%f\t%f\n', edge_plus(ii,:));
  end
  fclose(fp);
end